import 'dart:typed_data';
import 'package:vid/us/vid_us_data_reader.dart';
import 'package:vid/us/vid_us_image.dart';
import 'package:vid/us/vid_us_probe.dart';

enum VidUsImageFormat {
  Jpeg,
  Png,
  H264,
  Zip,
  Diff,
}

///Only suport read for now.
///Not suport carotid3d for now.
class VidUsImageData {
  late VidUsDataReader _reader;
  late List<int> _imagePositionList;
  late int _version;
  late int _imageCount;
  late VidUsProbe _probe;
  late VidUsImageFormat _imageFormat;
  late Uint8List _extendedData;

  final String header = "VINNO IMAGE DATA";

  /// Gets the version of this image data.
  int get version => _version;

  /// Gets the image count of this image data.
  int get imageCount => _imageCount;

  /// Gets the probe information.
  VidUsProbe get probe => _probe;

  /// Gets the image format of this image data.
  VidUsImageFormat get imageFormat => _imageFormat;

  /// Gets or sets the extended data.
  Uint8List get extendedData => _extendedData;

  /// Create a VINNO Image Data.
  VidUsImageData(Uint8List data) {
    _reader = new VidUsDataReader(data);
    var header = _reader.readString();
    if (header != header) {
      throw new Exception("The input data is not a VID data.");
    }
    _version = _reader.readInt();
    //Get probe info
    var probeData = _reader.readBytes();
    _probe = VidUsProbe.fromBytes(probeData);
    _imageFormat = VidUsImageFormat.values[_reader.readInt()];
    _extendedData = _reader.readBytes();
    //Get the index list.
    _imagePositionList = _reader.readLongs();
    _imageCount = _imagePositionList.length;
  }

  /// Get one image from the vid.
  VidUsImage getImage(int index) {
    if (index >= _imageCount || index < 0) {
      throw new Exception("Can not find image Data");
    }
    //Jump to image.
    var imageData = _reader.readBytes(_imagePositionList[index]);
    return VidUsImage.fromBytes(imageData);
  }
}
